'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global translate, changeTextDirection, Overlay */

// Requires overlay.js and jquery

// Highlight DOM elements with an overlayed box, similar to Webkit's inspector.
// Creates an absolute-positioned div that is translated & scaled following
// mousemove events. Holds a pointer to target DOM element.
function Highlighter() {
  let target = null;
  let enabled = false;
  let then = Date.now();
  let $box = $("<div class='adblock-highlight-node'></div>");
  $box.appendTo('body');

  function handler(e) {
    let el = e.target;
    const now = Date.now();
    if (now - then < 25) {
      return;
    }

    then = now;
    if (el === $box[0]) {
      $box.hide();
      el = document.elementFromPoint(e.clientX, e.clientY);
    }

    if (el === target) {
      $box.show();
      return;
    }

    if (el === document.body || el.className === 'adblock-killme-overlay') {
      $box.hide();
      return;
    }

    const $el = $(el);
    [target] = $el;
    const offset = $el.offset();
    $box.css({
      height: $el.outerHeight(),
      width: $el.outerWidth(),
      left: offset.left,
      top: offset.top,
    });
    $box.show();
  }

  this.getCurrentNode = function getNode(el) {
    return el === $box[0] ? target : el;
  };

  this.enable = function enableFunction() {
    if ($box && !enabled) {
      $('body').bind('mousemove', handler);
    }

    enabled = true;
  };

  this.disable = function disableFunction() {
    if ($box && enabled) {
      $box.hide();
      $('body').unbind('mousemove', handler);
    }

    enabled = false;
  };

  this.destroy = function destroyFunction() {
    this.disable();
    if ($box) {
      $box.remove();
      $box = null;
    }
  };
}

// Class that watches the whole page for a click, including iframes and
// objects.  Shows a modal while doing so.
function ClickWatcher() {
  this.callbacks = { cancel: [], click: [] };
  this.clickedElement = null;
  this.highlighter = new Highlighter();
}

ClickWatcher.prototype.cancel = function cancelFunction(callback) {
  this.callbacks.cancel.push(callback);
};

ClickWatcher.prototype.click = function clickFunction(callback) {
  this.callbacks.click.push(callback);
};

ClickWatcher.prototype.fire = function fireFunction(eventName, arg) {
  const callbacks = this.callbacks[eventName];
  for (let i = 0; i < callbacks.length; i += 1) {
    callbacks[i](arg);
  }
};

ClickWatcher.prototype.show = function showFunction() {
  const localThis = this;
  const wait = $('<div style="white-space: pre-wrap;" ></div>')
    .text(translate('findingads'))
    .dialog({
      classes: {
        'ui-dialog': 'adblock-blacklist-dialog',
      },
      closeText: '',
      position: { my: 'left top', at: 'left+10% top+10%', of: window },
      height: 120,
      minHeight: 120,
      title: translate('blockanadtitle'),
    });
  changeTextDirection($('body .adblock-blacklist-dialog'));

  // setTimeout to give 'wait' a chance to display
  window.setTimeout(() => {
    localThis.ui = localThis.build_ui();
    wait.dialog('close');
    wait.remove();
    localThis.ui.dialog('open');
    localThis.highlighter.enable();
  }, 10);
};

// Called externally to close ClickWatcher.  Doesn't cause any events to
// fire.
ClickWatcher.prototype.close = function closeFunction() {
  // Delete our event listeners so we don't fire any cancel events
  this.callbacks.cancel = [];
  if (this.ui) {
    this.ui.dialog('close');
  }
};

// The dialog is closing, either because the user clicked cancel, or the
// close button, or because they clicked an item.
ClickWatcher.prototype.onClose = function onCloseFunction() {
  if (!this.clickedElement) {
    // User clicked Cancel button or X
    this.fire('cancel');
  } else {
    // User clicked a page item
    this.fire('click', this.clickedElement);
  }

  this.highlighter.destroy();
};

ClickWatcher.prototype.build_ui = function buildUiFunction() {
  const localThis = this;

  function clickCatch(element) {
    localThis.clickedElement = localThis.highlighter.getCurrentNode(element);

    // localThis.clickedElement = element;
    localThis.ui.dialog('close');
    return false;
  }

  function clickCatchThis() {
    return clickCatch(this);
  }

  // Most things can be blacklisted with a simple click handler.
  $('body').on(
    'click', '.adblock-killme-overlay, .adblock-highlight-node',
    clickCatchThis,
  );

  // Since iframes that will get clicked will almost always be an entire
  // ad, and I *really* don't want to figure out inter-frame communication
  // so that the blacklist UI's slider works between multiple layers of
  // iframes... just overlay iframes and treat them as a giant object.
  $('object,embed,iframe,[onclick]:empty')
    .each((i, el) => {
      // Don't add overlay's for hidden elements
      if (el.style
            && el.style.display === 'none') {
        return;
      }
      const killmeOverlay = new Overlay({
        dom_element: el,
        click_handler: clickCatch,
      });
      killmeOverlay.display();
    });

  const btn = {};

  const page = $('<div></div>')
    .text(translate('clickthead'))
    .append('<br/><br/>')
    .dialog({
      classes: {
        'ui-dialog': 'adblock-blacklist-dialog',
      },
      closeText: '',
      position: { my: 'left top', at: 'left+10% top+10%', of: window },
      width: 400,
      minHeight: 125,
      autoOpen: false,
      title: translate('blockanadtitle'),
      buttons: btn,
      close() {
        $('body').off(
          'click',
          '.adblock-killme-overlay, .adblock-highlight-node', clickCatchThis,
        );
        Overlay.removeAll();
        localThis.onClose();
        page.remove();
      },

      drag() {
        localThis.highlighter.disable();
      },
    });

  btn[translate('buttoncancel')] = function buttonCancelFunction() {
    $('.adblock-ui-stylesheet').remove();
    page.dialog('close');
  };

  page.dialog('widget')
    .bind('mouseenter', () => {
      localThis.highlighter.disable();
    })
    .bind('mouseleave', () => {
      localThis.highlighter.enable();
    });

  changeTextDirection($('body .adblock-blacklist-dialog'));
  return page;
};

// required return value for tabs.executeScript
/* eslint-disable-next-line no-unused-expressions */
'';

// # sourceURL=/uiscripts/blacklisting/clickwatcher.js
